
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#ifdef __TURBOC__
#  include <alloc.h>
#else
#  define coreleft() 0L
#endif

#if defined(__STDC__) || defined(__MSDOS__)
#  define OF(args)  args
#else
#  define OF(args)  ()
#endif

#if defined(sparc) && defined(__GNUC__) && !defined(INT_MULT)
#  define FLOAT_MULT
  /* with gcc, floating point multiply is faster than integer multiply */
#endif
#ifdef FLOAT_MULT
    typedef double cum_data;
#else
    typedef unsigned long cum_data;
#endif

#define VERSION 1
/* VERSION is used to check the format of compressed .fra files.
 * This number must be incremented if you change the format.
 */

#define GREY_LEVELS 255

/* ==========================================================================
 * De-quantize an integer value in the range 0 .. imax to the range 0.0 .. max
 * while preserving the mapping 0 -> 0.0 and imax -> max.
 */
#define dequantize(value, max, imax) ((double)(value)*(max)/(double)imax)

typedef unsigned char image_data; /* pixel values are 0..255 */

void fatal_error OF((char *s));
void *xalloc OF((unsigned size));
void **allocate OF((int rows, int columns, int elem_size));
int  bitlength OF((unsigned long val));

#ifdef DEBUG
#  define Trace(x) {printf x ; fflush(stdout);}
#  ifdef verbose
#    define Tracev(x) Trace(x)
#  else
#    define Tracev(x)
#  endif
#  define Assert(cond, msg) {if (!(cond)) fatal_error(msg);}
   long alloc_total = 0L;
#else
#  define Trace(x)
#  define Tracev(x)
#  define Assert(cond, msg)
#endif

void fatal_error(s)
    char *s;
{
    Trace(("alloc_total %ld", alloc_total));
    Trace(("coreleft %ld\n", coreleft()));
    printf("\n%s\n", s);
    exit(1);
}

void *xalloc(size)
    unsigned size;
{
    void *p = malloc(size);
#ifdef DEBUG
    alloc_total += (long)size;
    Tracev(("alloc %u, total %ld\n", size, total));
#endif
    if (p == NULL) fatal_error("insufficient memory");
    return p;
}

/* Allocate a two dimensional array. For portability to 16-bit
 * architectures with segments limited to 64K, we allocate one
 * array per row, so the two dimensional array is allocated
 * as an array of arrays.
 */
void **allocate(rows, columns, elem_size)
    int rows;      /* number of rows */
    int columns;   /* number of columns */
    int elem_size; /* element size */
{
    int row;
    void **array = (void**)xalloc(rows * sizeof(void *));
    Assert((long)columns*elem_size < 65535L, "too large");
    for (row = 0; row < rows; row++) {
        array[row] = (void*)xalloc(columns * elem_size);
    }
    return array;
}

/* Return the number of bits needed to represent an integer:
 * 0 to 1 -> 1,
 * 2 to 3 -> 2,
 * 3 to 7 -> 3, etc...
 * This function could be made faster with a lookup table.
 */
int  bitlength(val)
    unsigned long val;
{
    int bits = 1;
    
    if (val > 0xffff) bits += 16, val >>= 16;
    if (val > 0xff)   bits += 8,  val >>= 8;
    if (val > 0xf)    bits += 4,  val >>= 4;
    if (val > 0x3)    bits += 2,  val >>= 2;
    if (val > 0x1)    bits += 1;
    return bits;
}
